<?php

namespace Drupal\reinstall\Plugin\migrate\source;

/**
 * Class SimpleSourceTrait provides shared features for entity field handling.
 */
trait SimpleSourceTrait {

  /**
   * The entity_type.bundle.info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $sstEntityTypeBundleInfo;

  /**
   * The entity_field.manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $sstEntityFieldManager;

  /**
   * The name of the entity type being handled.
   *
   * @var string
   */
  protected $sstEntityType;

  /**
   * The entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $sstEntityTypeManager;

  /**
   * Instance cache for the entity ID names, to avoid fetching them every time.
   *
   * @var array|null
   */
  protected $sstIds = NULL;

  /**
   * {@inheritdoc}
   */
  public function fields() {
    $bundles = $this->getEntityTypeBundleInfo()
      ->getBundleInfo($typeName = $this->getEntityType());
    $rows = [];

    foreach ($bundles as $bundleName => $bundle) {
      $fieldDefinitions = $this->getEntityFieldManager()
        ->getFieldDefinitions($typeName, $bundleName);
      foreach ($fieldDefinitions as $fieldName => $fieldDefinition) {
        $rows[$fieldName][$bundleName] = $fieldDefinition->getLabel();
      }
    }

    $fields = [];
    $singleBundle = count($bundles) === 1;
    foreach ($rows as $fieldName => $labels) {
      if ($singleBundle) {
        $fields[$fieldName] = reset($labels);
        continue;
      }

      if (count(array_unique($labels)) === 1) {
        $fields[$fieldName] = reset($labels);
        continue;
      }

      $ret = [];
      ksort($labels);
      foreach ($labels as $ct => $label) {
        $ret[] = $this->t('@ct: @label', ['@ct' => $ct, '@label' => $label]);
      }
      $fields[$fieldName] = implode(', ', $ret);
    }

    ksort($fields);
    return $fields;
  }

  /**
   * Getter for the entity_type.bundle.info service.
   *
   * @return \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   *   The service.
   */
  protected function getEntityTypeBundleInfo() {
    if (!isset($this->sstEntityTypeBundleInfo)) {
      $this->sstEntityTypeBundleInfo = \Drupal::service('entity_type.bundle.info');
    }

    return $this->sstEntityTypeBundleInfo;
  }

  /**
   * Getter for the entity_field.manager service.
   *
   * @return \Drupal\Core\Entity\EntityFieldManagerInterface|mixed
   *   The service.
   */
  protected function getEntityFieldManager() {
    if (!isset($this->sstEntityFieldManager)) {
      $this->sstEntityFieldManager = \Drupal::service('entity_field.manager');
    }

    return $this->sstEntityFieldManager;
  }

  /**
   * Getter for the current entity type.
   *
   * @return string
   *   The machine name of the type.
   */
  protected function getEntityType() {
    assert(isset($this->sstEntityType));
    return $this->sstEntityType;
  }

  /**
   * Getter for the entity_type.manager service.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface|mixed
   *   The service.
   */
  protected function getEntityTypeManager() {
    if (!isset($this->sstEntityTypeManager)) {
      $this->sstEntityTypeManager = \Drupal::service('entity_type.manager');
    }

    return $this->sstEntityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public function getIds() {
    if (is_array($this->sstIds)) {
      return $this->sstIds;
    }

    $typeName = $this->getEntityType();
    $typeDefinition = $this->getEntityTypeManager()->getDefinition($typeName);

    $fieldManager = $this->getEntityFieldManager();
    $definitions = $fieldManager->getBaseFieldDefinitions($typeName);

    $keys = $this->configuration['keys'] ?? ['id'];
    $ids = [];
    foreach ($keys as $key) {
      $keyName = $typeDefinition->getKey($key);
      assert(!empty($keyName));
      assert(isset($definitions[$keyName]));
      $idDefinition = $definitions[$keyName];
      $idType = $idDefinition->getType();
      $ids[$keyName] = ['type' => $idType];
    }

    if (!empty($this->configuration['raw'])) {
      $keys = array_keys($ids);
      foreach ($keys as $key) {
        $ids[ReinstallSourceBase::INJECTED_ID_PREFIX . $key] = $ids[$key];
        unset($ids[$key]);
      }
    };

    $this->sstIds = $ids;
    return $this->sstIds;
  }

}
